"""
A set partitioning model of a wedding seating problem

Authors: Stuart Mitchell 2009
"""
from time import time
#time this application
start_time = time()
import coinor.pulp as pulp
from random import seed

seed(0)

def unhappiness(table):
    """
    Find the unhappiness of the table
    by calculating the maximum distance between the letters
    """
    if len(table) > 1:
        result = max(ord(guest_b) - ord(guest_a)
                     for guest_a in table
                     for guest_b in table
                     if ord(guest_a) < ord(guest_b))
    else:
        result = 27
    return result

def solve_problem(guests, max_table_size, max_tables):
    #time this application
    start_time = time()
    #create list of all possible tables
    possible_tables = [tuple(c) for c in pulp.allcombinations(guests, 
                                            max_table_size)]
    gen_time = time() - start_time
    start_time = time()
    #create a binary variable to state that a table setting is used
    x = pulp.LpVariable.dicts('table', possible_tables, 
                                lowBound = 0,
                                upBound = 1,
                                cat = pulp.LpInteger)

    seating_model = pulp.LpProblem("Wedding Seating Model", pulp.LpMinimize)

    seating_model += sum([unhappiness(table) * x[table] for table in possible_tables])

    #specify the maximum number of tables
    seating_model += sum([x[table] for table in possible_tables]) <= max_tables, "Max_tables"

    #A guest must seated at one and only one table
    for guest in guests:
        seating_model += sum([x[table] for table in possible_tables
                                    if guest in table]) == 1, "Must_seat_%s"%guest

    seating_model.solve()
    print seating_model.objective.value(), sum([unhappiness(table) * x[table].varValue for table in possible_tables])
    end_time = time()-start_time
    return gen_time, end_time, seating_model.objective.value(), x, possible_tables

if __name__ == '__main__':
    gen_time, end_time, obj, x, possible_tables = solve_problem(guests, max_table_size, max_tables)
    print "The choosen tables are out of a total of %s:"%len(possible_tables)
    for table in possible_tables:
        if x[table].value() == 1.0:
            print table, unhappiness(table)

    print 'Total solution time for SPP formulation %s seconds'%end_time
